@@ -1,6 +1,6 @@
 // main.cpp
 
-// Copyright (C) 2011-2016 by Werner Lemberg.
+// Copyright (C) 2011-2017 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -305,6 +305,11 @@ show_help(bool
 #ifndef BUILD_GUI
 "      --debug                print debugging information\n"
 #endif
+"  -a, --stem-width-mode=S    select stem width mode for grayscale, GDI\n"
+"                             ClearType, and DW ClearType, where S is a\n"
+"                             string of three letters with possible values\n"
+"                             `n' for natural, `q' for quantized, and `s'\n"
+"                             for strong (default: qsq)\n"
 "  -c, --composites           hint glyph composites also\n"
 "  -d, --dehint               remove all hints\n"
 "  -D, --default-script=S     set default OpenType script (default: latn)\n"
@@ -346,11 +351,6 @@ show_help(bool
 #endif
 "  -v, --verbose              show progress information\n"
 "  -V, --version              print version information and exit\n"
-"  -w, --strong-stem-width=S  use strong stem width routine for modes S,\n"
-"                             where S is a string of up to three letters\n"
-"                             with possible values `g' for grayscale,\n"
-"                             `G' for GDI ClearType, and `D' for\n"
-"                             DirectWrite ClearType (default: G)\n"
 "  -W, --windows-compatibility\n"
 "                             add blue zones for `usWinAscent' and\n"
 "                             `usWinDescent' to avoid clipping\n"
@@ -465,6 +465,8 @@ show_help(bool
 "\n"
 "  [<font idx>] <script> <feature> @ <glyph ids>\n"
 "\n"
+"  [<font idx>] <script> <feature> w <stem widths>\n"
+"\n"
 "  [<font idx>] <glyph id> l|r <points> [(<left offset>,<right offset>)]\n"
 "\n"
 "  [<font idx>] <glyph id> n <points>\n"
@@ -472,7 +474,8 @@ show_help(bool
 "  [<font idx>] <glyph id> t|p <points> [x <shift>] [y <shift>] @ <ppems>\n"
 "\n"
 "<font idx> is the current subfont, <glyph id> is a glyph name or index,\n"
-"<glyph ids> is a set of <glyph id>s, <shift> is a real number in px,\n"
+"<glyph ids> is a set of <glyph id>s, <stem widths> is an unordered set of\n"
+"integer stem widths in font units, <shift> is a real number in px,\n"
 "<points> and <ppems> are integer ranges as with option `-X'.\n"
 "\n"
 "<script> and <feature> are four-letter tags that define a style\n"
@@ -495,6 +498,7 @@ show_help(bool
   }
   fprintf(handle,
 "\n"
+"`w' assigns stem widths to a style; the first value sets the default.\n"
 "`l' (`r') creates one-point segments with direction left (right).\n"
 "<left offset> and <right offset> specify offsets (in font units)\n"
 "relative to the corresponding points to give the segments a length.\n"
@@ -504,8 +508,9 @@ show_help(bool
 "`#' starts a line comment, which gets ignored.\n"
 "Empty lines are ignored, too.\n"
 "\n"
-"Key letters `l', `r', `n', `p', `t', `x', and `y' have the verbose aliases\n"
-"`left', `right', `nodir', `point', `touch', `xshift', and `yshift'.\n"
+"Key letters `l', `r', `n', `p', `t', `w', `x', and `y'\n"
+"have the verbose aliases `left', `right', `nodir', `point', `touch',\n"
+"`width', `xshift', and `yshift', respectively.\n"
 #endif
 "\n"
 #ifdef BUILD_GUI
@@ -534,7 +539,7 @@ show_version()
 #else
 "ttfautohint " VERSION "\n"
 #endif
-"Copyright (C) 2011-2016 Werner Lemberg <wl@gnu.org>.\n"
+"Copyright (C) 2011-2017 Werner Lemberg <wl@gnu.org>.\n"
 "License: FreeType License (FTL) or GNU GPLv2.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law.\n");
@@ -703,9 +708,13 @@ main(int argc,
   bool have_increase_x_height = false;
   bool have_fallback_stem_width = false;
 
-  bool gray_strong_stem_width = false;
-  bool gdi_cleartype_strong_stem_width = true;
-  bool dw_cleartype_strong_stem_width = false;
+  int gray_stem_width_mode = TA_STEM_WIDTH_MODE_QUANTIZED;
+  int gdi_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_STRONG;
+  int dw_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_QUANTIZED;
+
+#ifndef BUILD_GUI
+  bool have_option_w = false;
+#endif
 
   bool ignore_restrictions = false;
   bool windows_compatibility = false;
@@ -801,6 +810,7 @@ main(int argc,
       {"reference", required_argument, NULL, 'R'},
       {"reference-index", required_argument, NULL, 'Z'},
 #endif
+      {"stem-width-mode", required_argument, NULL, 'a'},
       {"strong-stem-width", required_argument, NULL, 'w'},
       {"symbol", no_argument, NULL, 's'},
       {"ttfa-table", no_argument, NULL, 't'},
@@ -844,9 +854,9 @@ main(int argc,
     int option_index;
     int c = getopt_long_only(argc, argv,
 #ifdef BUILD_GUI
-                             "cdD:f:F:G:hH:iIl:npr:sStvVw:Wx:X:",
+                             "a:cdD:f:F:G:hH:iIl:npr:sStvVw:Wx:X:",
 #else
-                             "cdD:f:F:G:hH:iIl:m:npr:R:sStTvVw:Wx:X:Z:",
+                             "a:cdD:f:F:G:hH:iIl:m:npr:R:sStTvVw:Wx:X:Z:",
 #endif
                              long_options, &option_index);
     if (c == -1)
@@ -854,6 +864,67 @@ main(int argc,
 
     switch (c)
     {
+    case 'a':
+      if (strlen(optarg) != 3)
+      {
+        fprintf(stderr, "Stem width mode string must consist of exactly"
+                        " three letters\n");
+        exit(EXIT_FAILURE);
+      }
+
+      switch (optarg[0])
+      {
+      case 'n':
+        gray_stem_width_mode = TA_STEM_WIDTH_MODE_NATURAL;
+        break;
+      case 'q':
+        gray_stem_width_mode = TA_STEM_WIDTH_MODE_QUANTIZED;
+        break;
+      case 's':
+        gray_stem_width_mode = TA_STEM_WIDTH_MODE_STRONG;
+        break;
+      default:
+        fprintf(stderr, "Stem width mode letter for grayscale rendering"
+                        " must be `n', `q', or `s'\n");
+        exit(EXIT_FAILURE);
+      }
+
+      switch (optarg[1])
+      {
+      case 'n':
+        gdi_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_NATURAL;
+        break;
+      case 'q':
+        gdi_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_QUANTIZED;
+        break;
+      case 's':
+        gdi_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_STRONG;
+        break;
+      default:
+        fprintf(stderr, "Stem width mode letter for GDI ClearType rendering"
+                        " must be `n', `q', or `s'\n");
+        exit(EXIT_FAILURE);
+      }
+
+      switch (optarg[2])
+      {
+      case 'n':
+        dw_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_NATURAL;
+        break;
+      case 'q':
+        dw_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_QUANTIZED;
+        break;
+      case 's':
+        dw_cleartype_stem_width_mode = TA_STEM_WIDTH_MODE_STRONG;
+        break;
+      default:
+        fprintf(stderr, "Stem width mode letter for DW ClearType rendering"
+                        " must be `n', `q', or `s'\n");
+        exit(EXIT_FAILURE);
+      }
+
+      break;
+
     case 'c':
       hint_composites = true;
       break;
@@ -964,9 +1035,18 @@ main(int argc,
       break;
 
     case 'w':
-      gray_strong_stem_width = strchr(optarg, 'g') ? true : false;
-      gdi_cleartype_strong_stem_width = strchr(optarg, 'G') ? true : false;
-      dw_cleartype_strong_stem_width = strchr(optarg, 'D') ? true : false;
+      gray_stem_width_mode = strchr(optarg, 'g')
+                               ? TA_STEM_WIDTH_MODE_STRONG
+                               : TA_STEM_WIDTH_MODE_QUANTIZED;
+      gdi_cleartype_stem_width_mode = strchr(optarg, 'G')
+                                        ? TA_STEM_WIDTH_MODE_STRONG
+                                        : TA_STEM_WIDTH_MODE_QUANTIZED;
+      dw_cleartype_stem_width_mode = strchr(optarg, 'D')
+                                       ? TA_STEM_WIDTH_MODE_STRONG
+                                       : TA_STEM_WIDTH_MODE_QUANTIZED;
+#ifndef BUILD_GUI
+      have_option_w = true;
+#endif
       break;
 
     case 'W':
@@ -1118,6 +1198,11 @@ main(int argc,
     }
   }
 
+  if (have_option_w)
+    fprintf(stderr,
+            "Warning: Option `-w' is deprecated!"
+            "  Use option `-a' instead\n");
+
   if (!isatty(fileno(stderr)) && !debug)
     setvbuf(stderr, (char*)NULL, _IONBF, BUFSIZ);
 
@@ -1350,9 +1435,9 @@ main(int argc,
   info_data.hinting_range_max = hinting_range_max;
   info_data.hinting_limit = hinting_limit;
 
-  info_data.gray_strong_stem_width = gray_strong_stem_width;
-  info_data.gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
-  info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
+  info_data.gray_stem_width_mode = gray_stem_width_mode;
+  info_data.gdi_cleartype_stem_width_mode = gdi_cleartype_stem_width_mode;
+  info_data.dw_cleartype_stem_width_mode = dw_cleartype_stem_width_mode;
 
   info_data.windows_compatibility = windows_compatibility;
   info_data.adjust_subglyphs = adjust_subglyphs;
@@ -1395,8 +1480,8 @@ main(int argc,
     TTF_autohint("in-file, out-file, control-file,"
                  "reference-file, reference-index, reference-name,"
                  "hinting-range-min, hinting-range-max, hinting-limit,"
-                 "gray-strong-stem-width, gdi-cleartype-strong-stem-width,"
-                 "dw-cleartype-strong-stem-width,"
+                 "gray-stem-width-mode, gdi-cleartype-stem-width-mode,"
+                 "dw-cleartype-stem-width-mode,"
                  "progress-callback, progress-callback-data,"
                  "error-callback, error-callback-data,"
                  "info-callback, info-post-callback, info-callback-data,"
@@ -1409,8 +1494,8 @@ main(int argc,
                  in, out, control,
                  reference, reference_index, reference_name,
                  hinting_range_min, hinting_range_max, hinting_limit,
-                 gray_strong_stem_width, gdi_cleartype_strong_stem_width,
-                 dw_cleartype_strong_stem_width,
+                 gray_stem_width_mode, gdi_cleartype_stem_width_mode,
+                 dw_cleartype_stem_width_mode,
                  progress_func, &progress_data,
                  err_func, &error_data,
                  info_func, info_post_func, &info_data,
@@ -1480,8 +1565,8 @@ main(int argc,
 
     Main_GUI dummy(alternative_layout,
                    hinting_range_min, hinting_range_max, hinting_limit,
-                   gray_strong_stem_width, gdi_cleartype_strong_stem_width,
-                   dw_cleartype_strong_stem_width, increase_x_height,
+                   gray_stem_width_mode, gdi_cleartype_stem_width_mode,
+                   dw_cleartype_stem_width_mode, increase_x_height,
                    x_height_snapping_exceptions_string, fallback_stem_width,
                    ignore_restrictions, windows_compatibility, adjust_subglyphs,
                    hint_composites, no_info, detailed_info,
@@ -1500,8 +1585,8 @@ main(int argc,
 
   Main_GUI gui(alternative_layout,
                hinting_range_min, hinting_range_max, hinting_limit,
-               gray_strong_stem_width, gdi_cleartype_strong_stem_width,
-               dw_cleartype_strong_stem_width, increase_x_height,
+               gray_stem_width_mode, gdi_cleartype_stem_width_mode,
+               dw_cleartype_stem_width_mode, increase_x_height,
                x_height_snapping_exceptions_string, fallback_stem_width,
                ignore_restrictions, windows_compatibility, adjust_subglyphs,
                hint_composites, no_info, detailed_info,
